@@ -2,8 +2,7 @@
 import queue
 import time
 import traceback
-import multiprocessing as mp
-from typing import Any, Dict, List, Optional
+from typing import Any, List, Optional
 from dataclasses import dataclass
 from threading import Thread, Condition, Event
 import uuid
@@ -17,22 +16,8 @@ from .config import BatchingConfig
 EVENT_KEY_PREFIX = 'pyraidynb'
 
 
-@dataclass
-class MsgSend:
-    id: str
-    item: Any
-
-
-@dataclass
-class MsgRecv:
-    id: str
-    res: Any
-    ex: Optional[Exception]
-    
-
 class ItemFuture:
-    def __init__(self, uid: str, cancellation_token: Optional[Event]=None):
-        self._uid = uid
+    def __init__(self, cancellation_token: Optional[Event]=None):
         self._condition = Condition()
         self._flag_done = False
         self._flag_cancelled = False
@@ -108,6 +93,11 @@ class ItemFuture:
             self._condition.notify_all()
 
 
+@dataclass
+class ItemMessage:
+    future: ItemFuture
+    item: Any
+    create_ts: float
     
 @dataclass
 class RequestCorrelation:
@@ -159,15 +149,11 @@ class DynamicBatchModel:
         assert self.idle_batch_size <= self.max_batch_size, f'Require PYRAISDK_IDLE_BATCH_SIZE ({self.idle_batch_size}) <= PYRAISDK_MAX_BATCH_SIZE ({self.max_batch_size})'
 
         self.model = model
-        self.futuredict: Dict[str, ItemFuture] = {}
-        self.alive = mp.Event()
-        self.q_send: mp.Queue[MsgSend] = mp.Queue()
-        self.q_recv: mp.Queue[MsgRecv] = mp.Queue()
-        
-        self.worker = mp.Process(target=self._worker_run, daemon=True)
-        self.reciever = Thread(target=self._reciever_run, daemon=True)
+        self.alive = True
+        self.q: queue.Queue[ItemMessage] = queue.Queue()
+        self.worker = Thread(target=self._worker_run, daemon=True)
         self.worker.start()
-        self.reciever.start()
+        
     
     def predict(
         self,
@@ -202,11 +188,9 @@ class DynamicBatchModel:
             # get futures & put messsage into queue
             futurelist: List[ItemFuture] = []
             for item in items:
-                uid = str(uuid.uuid4())
-                future = ItemFuture(uid, cancellation_token)
-                self.futuredict[uid] = future
-                msg = MsgSend(uid, item)
-                self.q_send.put_nowait(msg)
+                future = ItemFuture(cancellation_token)
+                msg = ItemMessage(future, item, ts_start)
+                self.q.put_nowait(msg)
                 futurelist.append(future)
             
             # get results
@@ -221,10 +205,6 @@ class DynamicBatchModel:
                     result = future.get_result(timeout=time_left)
                 except PredictTimeoutError:
                     future.set_cancelled()
-                    try:
-                        del self.futuredict[future._uid]
-                    except KeyError:
-                        pass
                     if raise_timeout:
                         raise
                     else:
@@ -277,7 +257,7 @@ class DynamicBatchModel:
 
     
     def close(self):
-        self.alive.set()
+        self.alive = False
         
         
     def __del__(self):
@@ -287,8 +267,7 @@ class DynamicBatchModel:
     def _worker_run(self):
         ''' If _worker_inner failed somehow, sleep some time and retry
         '''
-        self.model.load()
-        while not self.alive.is_set():
+        while self.alive:
             try:
                 self._worker_inner()
             except Exception:
@@ -303,14 +282,21 @@ class DynamicBatchModel:
             2. queue is empty && [batch size] >= self.idle_batch_size
             3. [now] - [last run time] >= self.max_batch_interval
         '''
-        batch: List[MsgSend] = []          # cached batch items
+        batch: List[ItemMessage] = []          # cached batch items
         last_run_time = time.perf_counter()    # last batch run time
         
-        while not self.alive.is_set():
+        while self.alive:
             # get from q without block & check empty
             try:
-                msg = self.q_send.get_nowait()
-                batch.append(msg)
+                msg = self.q.get_nowait()
+                # handle cancellation
+                if msg.future.is_external_ct_set():
+                    msg.future.set_cancelled()
+                    continue
+                elif msg.future.is_cancelled():
+                    continue
+                else:
+                    batch.append(msg)
             except queue.Empty:
                 msg = None
 
@@ -327,9 +313,15 @@ class DynamicBatchModel:
                 timeout = self.max_batch_interval - (time.perf_counter() - last_run_time)
                 if timeout > 0:
                     try:
-                        msg = self.q_send.get(block=True, timeout=timeout)
+                        msg = self.q.get(block=True, timeout=timeout)
                         # handle cancellation
-                        batch.append(msg)
+                        if msg.future.is_external_ct_set():
+                            msg.future.set_cancelled()
+                            continue
+                        if msg.future.is_cancelled():
+                            continue
+                        else:
+                            batch.append(msg)
                     except queue.Empty:
                         msg = None
 
@@ -342,7 +334,7 @@ class DynamicBatchModel:
                 last_run_time = time.perf_counter()
             
             
-    def _worker_run_batch(self, batch: List[MsgSend]):
+    def _worker_run_batch(self, batch: List[ItemMessage]):
         # preprocess & predict
         batch_track_id = str(uuid.uuid4())
         try:
@@ -371,45 +363,23 @@ class DynamicBatchModel:
 
         except Exception as e:
             for msg in batch:
-                mrecv = MsgRecv(msg.id, None, e)
-                self.q_recv.put_nowait(mrecv)
+                msg.future.set_excepted(e)
                 
             rlog.errorcf(batch_track_id, -1, e, f'{EVENT_KEY_PREFIX}: error running a batch')
             # items avg latency
             ts_end = time.perf_counter()
-            #latency_avg = sum(ts_end - msg.create_ts for msg in batch) / len(batch)
-            #rlog.event(f'{EVENT_KEY_PREFIX}_BatchItemsAvgLatency', 'error', latency_avg, corr_id=batch_track_id)
+            latency_avg = sum(ts_end - msg.create_ts for msg in batch) / len(batch)
+            rlog.event(f'{EVENT_KEY_PREFIX}_BatchItemsAvgLatency', 'error', latency_avg, corr_id=batch_track_id)
             return
         
         # set result for each
         for msg, result in zip(batch, output_predict):
-            mrecv = MsgRecv(msg.id, result, None)
-            self.q_recv.put_nowait(mrecv)            
+            msg.future.set_result(result)
 
         # items avg latency
         ts_end = time.perf_counter()
-        #latency_avg = sum(ts_end - msg.create_ts for msg in batch) / len(batch)
-        #rlog.event(f'{EVENT_KEY_PREFIX}_BatchItemsAvgLatency', 'success', latency_avg, corr_id=batch_track_id)
-
-    def _reciever_run(self):
-        while not self.alive.is_set():
-            try:
-                self._reciever_run_inner()
-            except Exception:
-                # unexpected error
-                traceback.print_exc()
-                time.sleep(1)
-    
-    def _reciever_run_inner(self):
-        while True:
-            msg: MsgRecv = self.q_recv.get()
-            future = self.futuredict.get(msg.id)
-            if future is None:
-                continue
-            if msg.ex:
-                future.set_excepted(msg.ex)
-            else:
-                future.set_result(msg.res)
+        latency_avg = sum(ts_end - msg.create_ts for msg in batch) / len(batch)
+        rlog.event(f'{EVENT_KEY_PREFIX}_BatchItemsAvgLatency', 'success', latency_avg, corr_id=batch_track_id)
 
 
 def get_request_correlation() -> RequestCorrelation:
