@@ -31,10 +31,11 @@ class AsyncEventLogger(EventLogger):
         self.unstruct_queue: mp.Queue[Event] = mp.Queue(maxsize=LOGGING_QUEUE_CAPACITY)
         self.struct_queue: mp.Queue[Event] = mp.Queue(maxsize=LOGGING_QUEUE_CAPACITY)
         self.backend_loop: Optional[_AsyncEventLoggerBackendLoop] = None
+        self.enable = True
     
     def log(self, level: LogLevel, msg: str, corr_id: str='', elem: int=-1):
         # check logger started
-        if self.backend_loop is None:
+        if self.backend_loop is None or not self.enable:
             return
         evt = UnstructuredEvent(
                 level = level,
@@ -53,7 +54,7 @@ class AsyncEventLogger(EventLogger):
 
     def event(self, key: str, code: str, numeric: float, detail: str='', corr_id: str='', elem: int=-1):
         # check logger started
-        if self.backend_loop is None:
+        if self.backend_loop is None or not self.enable:
             return
         evt = StructuredEvent(
             key = key,
