@@ -50,6 +50,10 @@ class SystemMetricsReporter:
         self.gpu_util_avg = Avarager()
         self.gpu_mem_util_avg = Avarager()
         
+        self.enable_cpu = True
+        self.enable_mem = True
+        self.enable_gpu = True
+        
         self.alive = True
         self.worker = Thread(target=self._worker_run, daemon=True)
         self.worker.start()
@@ -95,59 +99,65 @@ class SystemMetricsReporter:
 
     def _measure_process(self):
         # cpu
-        cpu_percent = psutil.cpu_percent()
-        self.cpu_util_avg.push(cpu_percent)
+        if self.enable_cpu:
+            cpu_percent = psutil.cpu_percent()
+            self.cpu_util_avg.push(cpu_percent)
 
         # memory
-        memory_percent = psutil.virtual_memory().percent
-        self.mem_util_avg.push(memory_percent)
+        if self.enable_mem:
+            memory_percent = psutil.virtual_memory().percent
+            self.mem_util_avg.push(memory_percent)
 
         # gpu
         # Only for the first time (partial first time) unable to get gpu info, log
         # will be pushed. To avoid frequently being disturbed by cpu error/warn logs.
-        gpu_ex = None
-        gpu_summary = None
-        try:
-            gpu_summary = get_gpu_summary()
-        except Exception as ex:
-            gpu_ex = ex
+        if self.enable_gpu:
+            gpu_ex = None
+            gpu_summary = None
+            try:
+                gpu_summary = get_gpu_summary()
+            except Exception as ex:
+                gpu_ex = ex
 
-        if gpu_ex:
-            # gpu error, only push log when succeeded last time
-            if self.gputil_success:
-                self.gputil_success = False
-                self.logger.errorcf('', -1, gpu_ex, f'{EVENT_KEY_PREFIX}: failed to get gpu info')
+            if gpu_ex:
+                # gpu error, only push log when succeeded last time
+                if self.gputil_success:
+                    self.gputil_success = False
+                    self.logger.errorcf('', -1, gpu_ex, f'{EVENT_KEY_PREFIX}: failed to get gpu info')
 
-        elif gpu_summary is None:
-            # no gpu, only push log when succeeded last time
-            if self.gputil_success:
-                self.gputil_success = False
-                self.logger.warnf(f'{EVENT_KEY_PREFIX}: no gpu available')
+            elif gpu_summary is None:
+                # no gpu, only push log when succeeded last time
+                if self.gputil_success:
+                    self.gputil_success = False
+                    self.logger.warnf(f'{EVENT_KEY_PREFIX}: no gpu available')
 
-        else:
-            # success
-            self.gputil_success = True
-            self.gpu_util_avg.push(gpu_summary.utilization)
-            self.gpu_mem_util_avg.push(gpu_summary.memory_utilization)
+            else:
+                # success
+                self.gputil_success = True
+                self.gpu_util_avg.push(gpu_summary.utilization)
+                self.gpu_mem_util_avg.push(gpu_summary.memory_utilization)
 
 
     def _report_process(self):
         # cpu
-        cpu_util_val = self.cpu_util_avg.pop()
-        if cpu_util_val is not None:
-            self.logger.event(f'{EVENT_KEY_PREFIX}_CpuUtilization', '', cpu_util_val)
+        if self.enable_cpu:
+            cpu_util_val = self.cpu_util_avg.pop()
+            if cpu_util_val is not None:
+                self.logger.event(f'{EVENT_KEY_PREFIX}_CpuUtilization', '', cpu_util_val)
         
         # memory
-        mem_util_val = self.mem_util_avg.pop()
-        if mem_util_val is not None:
-            self.logger.event(f'{EVENT_KEY_PREFIX}_MemoryUtilization', '', mem_util_val)
+        if self.enable_mem:
+            mem_util_val = self.mem_util_avg.pop()
+            if mem_util_val is not None:
+                self.logger.event(f'{EVENT_KEY_PREFIX}_MemoryUtilization', '', mem_util_val)
         
         # gpu
-        gpu_util_val = self.gpu_util_avg.pop()
-        if gpu_util_val is not None:
-            self.logger.event(f'{EVENT_KEY_PREFIX}_GpuUtilization', '', gpu_util_val)
-            
-        # gpu mem
-        gpu_mem_util_val = self.gpu_mem_util_avg.pop()
-        if gpu_mem_util_val is not None:
-            self.logger.event(f'{EVENT_KEY_PREFIX}_GpuMemoryUtilization', '', gpu_mem_util_val)
+        if self.enable_gpu:
+            gpu_util_val = self.gpu_util_avg.pop()
+            if gpu_util_val is not None:
+                self.logger.event(f'{EVENT_KEY_PREFIX}_GpuUtilization', '', gpu_util_val)
+                
+            # gpu mem
+            gpu_mem_util_val = self.gpu_mem_util_avg.pop()
+            if gpu_mem_util_val is not None:
+                self.logger.event(f'{EVENT_KEY_PREFIX}_GpuMemoryUtilization', '', gpu_mem_util_val)
